
/******************************************************************************
* MODULE     : display.cc
* DESCRIPTION: Displaying return values of evaluation
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <match.hh>
#include <pretty.hh>
namespace mmx {

bool debug_output= false;

#define DATA_BEGIN   ((char) 2)
#define DATA_END     ((char) 5)
#define DATA_ESCAPE  ((char) 27)

/******************************************************************************
* Displaying results
******************************************************************************/

void
evaluator_rep::display_one (value v) {
  (void) evaluate (list (MMX_LESSLESS_VAR, symbol ("mmout"), v));
}

static bool
redundant (generic t, list l) {
  if (nil (l)) return false;
  if (is_alias (car (l)) && (car (l) [1] == t)) return true;
  return redundant (t, cdr (l));
}

void
evaluator_rep::display (value v) {
  // cout << "v= " << v << "\n";
  if (v->type == ID_OSTREAM) return;
  if (is_and (v->type)) {
    list vals = cdr (as_list (v->val ));
    list types= cdr (as_list (v->type));
    list tl   = types;
    if (N(vals) != N(types)) {
      while (!nil (types)) {
	if (!redundant (car (types), tl)) {
	  value t (car (types), ID_CLASS);
	  generic res= evaluate (list (MMX_TRANSTYPE, v, t));
	  if (is_value (res)) display (as_value (res));
	  else { cout << "error: " << lf; display_one (t); }
	}
	types= cdr (types);
      }
    }
    else {
      while (!nil (types)) {
	if (!redundant (car (types), tl))
	  display (value (car (vals), car (types)));
	vals = cdr (vals );
	types= cdr (types);
      }
    }
  }
  else {
    if (debug_output) cout << v << lf;
    else {
      if (texmacs_mode) {
	display_one (v);
	cout << " " << DATA_BEGIN << "latex:\\rightflush" << DATA_END << " ";
	brown_flag= true;
	display_one (value (v->type, ID_CLASS));
	brown_flag= false;
	cout << lf;
      }
      else {
	display_one (v);
	cout << ": ";
	display_one (value (v->type, ID_CLASS));
	cout << lf;
      }
    }
  }
}

} // namespace mmx
