
/******************************************************************************
* MODULE     : mmx_shell.cc
* DESCRIPTION: Mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <readline/readline.h>
#include <readline/history.h>

#include <list.hh>
#include <file.hh>
#include <error.hh>
#include <parser.hh>
#include <evaluator.hh>
#include <pretty.hh>

using namespace mmx;

#define DATA_BEGIN   ((char) 2)
#define DATA_END     ((char) 5)
#define DATA_ESCAPE  ((char) 27)

static int counter= 0;

string
input () {
  counter++;
  if (texmacs_mode) {
    cout << DATA_BEGIN << "channel:prompt" << DATA_END;
    cout << DATA_BEGIN << "latex:" << "{\\red Mmx $\\gg$}" << DATA_END;
    cout << DATA_END;
    fflush (stdout);
    char in[1000];
    cin.getline (in, 1000);
    cout << DATA_BEGIN << "verbatim:";
    return in;
  }
  else {
    string prompt= as_string (counter) * "> ";
    char* _prompt= as_charp (prompt);
    char* line = readline (_prompt);
    if (line[0] != '\0') add_history (line);
    return line;
  }
}

void
error_report (generic g) {
  list errs= cdr (as_list (g));
  print_errors (errs);
}

void
output (generic g) {
  cout << blank;
  if (is_value (g)) mmx_display (as_value (g));
  else cout << g;
  cout << stroke;
}

int
main (int argc, char** argv) {
  bool interactive= false;
  int arg_nr= 0;
  if (argc == 2) {
    string option (argv[1]);
    texmacs_mode= (option == "--texmacs");
    debug_output= (option == "--debug");
    if (option == "--version") {
      cout << VERSION << "\n";
      exit (0);
    }
  }

  string s;
  if (texmacs_mode) cout << DATA_BEGIN << "verbatim:";
  cout << "------------------------------------------------------------\n";
  cout << "|:*)            Welcome to Mathemagix " << VERSION
       << "            (*:|\n";
  cout << "|      (c) 2001--2003  by Joris van der Hoeven et al.      |\n";
  cout << "| This software falls under the GNU General Public License |\n";
  cout << "|         It comes without any warranty whatsoever         |\n";
  cout << "------------------------------------------------------------\n";
  while (true) {
    // Get input
    if (arg_nr == 0) {
      arg_nr++;
      if (load (".:$MMX_LOAD_PATH", "boot.mmx", s)) {
	cout << "Couldn't open boot file boot.mmx\n";
	continue;
      }
      // cout << "Processing boot.mmx\n";
    }
    else if (arg_nr < argc) {
      string arg= argv[arg_nr++];
      if (arg == "--texmacs") continue;
      if (arg == "--debug") continue;
      insert_source (arg, "cmdline");
      if (load (arg, s)) {
	cout << "Couldn't open file " << arg << "\n";
	continue;
      }
      cout << "Processing " << arg << "\n";
    }
    else {
      s= input ();
      if ((s == "quit") || (s == "quit;")) break;
      if ((s == "exit") || (s == "exit;")) break;
      if ((s == "bye") || (s == "bye;")) break;
      if ((s == "") || (s == ";")) continue;
      if ((s[N(s)-1] != ';') && (s[N(s)-1] != '}')) s= s * ";";
      insert_source (s, "cmdline");
      interactive= true;
    }

    // Evaluate input
    list errors;
    generic g= parse (s, errors);
    if (N(errors)!=0) {
      print_errors (errors);
      continue;
    }
    g= mmx_evaluate (g);
    if (is_func (g, symbol ("errors!"))) error_report (g);
    else if (interactive && (g != VAL_NONE)) output (g);
  }
  return 0;
}
