
/******************************************************************************
* MODULE     : ext_concrete.cc
* DESCRIPTION: Generate extension code for concrete exported types
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extend.hh>
#include <ccpp.hh>
#include <pretty.hh>
#include <converter.hh>
namespace mmx {

static converter cvt;

/******************************************************************************
* Filtering declarations
******************************************************************************/

list
extender::filter (list l, list target_l) {
  // keep only those declarations which are exported
  if (nil (l)) return l;
  else {
    bool ok= false;
    generic var  = get_decl_var  (car (l));
    generic type = get_decl_type (car (l));
    if (export_env -> contains (var)) {
      generic given= export_env [var];
      proof p= cvt->convert (given, type, CV_NORMAL);
      ok= !nil (p);
    }
    if (ok) return cons (car (target_l), filter (cdr (l), cdr (target_l)));
    else return filter (cdr (l), cdr (target_l));
  }
}

/******************************************************************************
* Produce the generic code for a given type
******************************************************************************/

void
extender::produce_atomic_type (generic mmx_name, generic ccpp_name) {
  list l= filter (specialize (imports, mmx_name, false),
		  specialize (imports, geninst (), false));
  l= decl_as_ccpp (l, true);
  generate (make_struct (type_rep (ccpp_name), l, list (gentype_rep ())));
}

void
extender::produce_template_type (symbol name, list args) {
  (void) name; (void) args;
  // cout << "name= " << name << lf;
  // cout << "args= " << args << lf;
}

void
extender::produce_type (generic g) {
  if (!is_func (g, MMX_DEFINE, 2))
    error (g, "Expected a declaration");
  else if (!is_func (g[1], MMX_TYPE, 2))
    error (g, "Expected a declaration");
  else if (!is_symbol (g[2]))
    error (g[2], "Expected a symbol");
  else {
    symbol  name= as_symbol (g[2]);
    generic type= g[1][2];
    if (type == MMX_CLASS) produce_atomic_type (g[1][1], g[2]);
    else if (is_func (type, MMX_FUNCTION, 2) && (type[2] == MMX_CLASS)) {
      list l (type[1]);
      if (is_func (type[1], MMX_CROSS)) l= cdr (as_list (type[1]));
      produce_template_type (name, l);
    }
  }
}

void
extender::produce_type (list l) {
  if (!nil (l)) {
    produce_type (car (l));
    produce_type (cdr (l));
  }
}

} // namespace mmx
