
/******************************************************************************
* MODULE     : ext_generic.cc
* DESCRIPTION: Produce the generic extension code
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extend.hh>
#include <ccpp.hh>
#include <pretty.hh>
#include <quantor.hh>
namespace mmx {

/******************************************************************************
* Specializing list of imports to a given type
******************************************************************************/

generic
extender::specialize (generic g, generic to, bool virtual_flag) {
  if (!is_func (g, MMX_FORALL, 2))
    error (g, "Expected a 'forall' type");
  else if (!is_func (g[1], MMX_TYPE, 2))
    error (g[1], "Expected a typed quantification variable");
  else if (g[1][2] != MMX_CLASS)
    error (g[1][2], "Expected 'Class' type");
  else if (is_func (g[2], MMX_FORALL, 2))
    error (g, "Double quantifications not yet allowed");
  else if (!is_func (g[2], MMX_DEFINE, 2))
    error (g[2], "Expected a declaration");
  else if (!is_func (g[2][1], MMX_TYPE, 2))
    error (g[2], "Expected a declaration");
  else {
    generic qvar= g[1][1];
    generic body= g[2];
    generic ret = substitute (body, qvar, to);
    if (virtual_flag)
      ret= list (ret[0],
		 list (ret[1][0], ret[1][1], Virtual (ret[1][2])),
		 ret[2]);
    return ret;
  }
  return g;
}

list
extender::specialize (list l, generic to, bool virtual_flag) {
  if (nil (l)) return l;
  else return cons (specialize (car (l), to, virtual_flag),
		    specialize (cdr (l), to, virtual_flag));
}

/******************************************************************************
* Naming parameters in function types
******************************************************************************/

static symbol
parameter (int i) {
  return string ("arg_") * as_string (i);
}

static list
name_parameters (list l, int i) {
  if (nil (l)) return l;
  else return cons (Type (parameter (i), car (l)),
		    name_parameters (cdr (l), i+1));
}

static generic
name_parameters (generic g) {
  // name parameters in function type
  if (is_func (g, MMX_TYPE, 2))
    return list (g[0], g[1], name_parameters (g[2]));
  else if (is_func (g, MMX_FUNCTION, 2))
    return list (g[0], name_parameters (g[1]), g[2]);
  else if (is_func (g, MMX_CROSS))
    return cons (g[0], name_parameters (cdr (as_list (g)), 1));
  else return g;
}

/******************************************************************************
* Specializing list of imports to a given type
******************************************************************************/

generic
extender::type_as_ccpp (generic g) {
  // search the C/C++ name of a Mathemagix type
  if (type_conv -> contains (g)) return type_conv [g];
  else if (is_list (g)) {
    list l= as_list (g), r;
    while (!nil (l)) {
      r= cons (type_as_ccpp (car (l)), r);
      l= cdr (l);
    }
    return reverse (r);
  }
  else if ((g == MMX_CROSS) ||
	   (g == MMX_FUNCTION) ||
	   (g == MMX_POINTER) ||
	   (g == MMX_ARRAY) ||
	   (g == MMX_CLASS) ||
	   (g == MMX_BOOL) ||
	   (g == CCPP_CONST) ||
	   (g == CCPP_VIRTUAL) ||
	   (g == geninst ()))
    return g;
  else {
    int i;
    for (i=1; i<=max_type_args; i++)
      if (g == geninst (i))
	return g;
    warning (g, "Type could not be translated");
    return g;
  }
}

generic
extender::decl_as_ccpp (generic g, bool gen_flag) {
  if (!is_func (g, MMX_DEFINE, 2))
    error (g, "Expected a declaration");
  else if (!is_func (g[1], MMX_TYPE, 2))
    error (g, "Expected a declaration");
  else if (!is_symbol (g[2]))
    error (g[2], "Expected a symbol");
  else if (!gen_flag)
    return list (g[1][0], g[2], type_as_ccpp (g[1][2]));
  else return list (g[1][0], genmethod (g[2]), type_as_ccpp (g[1][2]));
  return g;
}

list
extender::decl_as_ccpp (list l, bool gen_flag) {
  if (nil (l)) return l;
  else return cons (decl_as_ccpp (car (l), gen_flag),
		    decl_as_ccpp (cdr (l), gen_flag));
}

/******************************************************************************
* Produce the gentype type. This type serves as a base type for
* all dynamic types which have been exported by the user.
******************************************************************************/

static list
arg_list (int start, int end) {
  if (start >= end) return list ();
  else return cons (parameter (start), arg_list (start+1, end));
}

void
extender::gentype_methods (generic g) {
  if (!is_func (g, MMX_TYPE, 2))
    error (g, "Expected a declaration");
  else if (!is_symbol (g[1]))
    error (g[1], "Expected a symbol");
  else if (!is_func (g[2], MMX_FUNCTION, 2))
    error (g[2], "Expected a function type");
  else if (!is_func (g[2][1], MMX_CROSS))
    error (g[2][1], "Expected a cross type");
  else {
    g= name_parameters (g);
    generic name= genmethod (g[1]);
    generic scoped= Scope (gentype_rep (), name);
    g= list (g[0], scoped, g[2]);
    generic fun = Arrow (curtype (1), name);
    generic _error= list (MMX_LESSLESS, symbol ("cerr"),
			  symbol ("\"Fatal error: unsupported method\\n\""));
    generic _exit = list (symbol ("exit"), symbol ("1"));
    generate (Define (g, Begin (_error, _exit)));
  }
}

void
extender::gentype_methods (list l) {
  if (!nil (l)) {
    gentype_methods (car (l));
    gentype_methods (cdr (l));
  }
}

void
extender::produce_gentype () {
  generate (Type (gentype_rep (), MMX_CLASS));
  generate (Type (geninst (), MMX_CLASS));
  generate (Typedef (gentype, Pointer (gentype_rep ())));
  list l= specialize (imports, geninst (), true);
  l= decl_as_ccpp (l, true);
  generic gc= Define (Type (gentype_rep (), Inline (Constructor (MMX_VOID))),
		      Begin ());
  generic gd= Define (Type (gentype_rep (), Inline (Virtual (Destructor ()))),
		      Begin ());
  l= cons (gc, gd, l);
  generate (make_struct (gentype_rep (), l));
  l= specialize (imports, geninst (), false);
  l= decl_as_ccpp (l);
  gentype_methods (l);
}

/******************************************************************************
* Produce the generic instance types, indexed by numbers 1, 2, ....
* These types are instances of dynamic types. curinst(i) holds
* the current type of geninst(i).
******************************************************************************/

void
extender::produce_geninst () {
  generate (make_blackbox_struct (geninst ()));
}

list
extender::abstract (int nr, list args, list types) {
  // convert to abstract type when necessary
  // FIXME: this does not work at the moment if one of the types
  // depends on geninst(nr), but is not equal to geninst(nr)
  if (nil (args)) return args;
  generic x= car (args);
  if (car (types) [2] == geninst (nr))
    x= list (abstract (nr), x);
  return cons (x, abstract (nr, cdr (args), cdr (types)));
}

void
extender::geninst_methods (int nr, generic g) {
  if (!is_func (g, MMX_TYPE, 2))
    error (g, "Expected a declaration");
  else if (!is_symbol (g[1]))
    error (g[1], "Expected a symbol");
  else if (!is_func (g[2], MMX_FUNCTION, 2))
    error (g[2], "Expected a function type");
  else if (!is_func (g[2][1], MMX_CROSS))
    error (g[2][1], "Expected a cross type");
  else {
    g= name_parameters (g);
    generic name= genmethod (g[1]);
    generic fun = Arrow (curtype (1), name);
    list    args= arg_list (1, N(g[2][1]));
    generic appl= cons (fun, abstract (nr, args, cdr (as_list (g[2][1]))));
    if (g[2][2] == geninst (nr)) appl= list (concrete (nr), appl);
    if (g[2][2] != MMX_VOID) appl= Return (appl);
    generate (Define (g, Begin (appl)));
  }
}

void
extender::geninst_methods (int nr, list l) {
  if (!nil (l)) {
    geninst_methods (nr, car (l));
    geninst_methods (nr, cdr (l));
  }
}

void
extender::produce_geninst (int nr) {
  generate (Type (curtype (nr), Static (gentype)));
  generate (make_blackbox_struct (geninst (nr)));
  generic con_t= Function (Type (symbol ("x"), geninst ()), geninst (nr));
  generic con_r= Begin (Return (list (geninst (nr),
				      Dot (symbol ("x"), symbol ("rep")))));
  generic con_d= Define (Type (concrete (nr), Inline (con_t)), con_r);
  generate (con_d);
  generic abs_t= Function (Type (symbol ("x"), geninst (nr)), geninst ());
  generic abs_r= Begin (Return (list (geninst (),
				      Dot (symbol ("x"), symbol ("rep")))));
  generic abs_d= Define (Type (abstract (nr), Inline (abs_t)), abs_r);
  generate (abs_d);
  list l= specialize (imports, geninst (nr), false);
  l= decl_as_ccpp (l);
  geninst_methods (nr, l);
}

} // namespace mmx
