
/******************************************************************************
* MODULE     : extend.cc
* DESCRIPTION: Automatically generate C++ extension code for generic objects
*              This should later be used in combination with mmx-glue
*              in order to produce glue code for templates
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extend.hh>
#include <ccpp.hh>
#include <pretty.hh>
#include <quantor.hh>
#include <converter.hh>
namespace mmx {

static converter cvt;

/******************************************************************************
* Error handling and code generation
******************************************************************************/

void
extender::error (generic where, string message) {
  cerr << "Error] ";
  raw_print (cerr, message);
  cerr << lf;
  cerr << "In   ] " << where << lf;
}

void
extender::warning (generic where, string message) {
  cerr << "Warning] ";
  raw_print (cerr, message);
  cerr << lf;
  cerr << "In     ] " << where << lf;
}

void
extender::generate (generic g) {
  output= cons (g, output);
}

/******************************************************************************
* Useful subroutines
******************************************************************************/

void
nice_print (generic text, list l) {
  cout << text << stroke << lf << indent;
  while (!nil (l)) {
    pretty_print (cout, car (l));
    cout << stroke << lf;
    l= cdr (l);
  }
  cout << unindent << "  " << lf;
}

void
nice_print (symbol text, list l) {
  nice_print ((generic) text, l);
}

generic
make_struct (generic name, list args, list inherit) {
  generic cl= MMX_CLASS;
  if (!nil (inherit)) cl= cons (CCPP_INHERIT, inherit);
  return Define (Type (name, cl), cons (CCPP_STRUCT, args));
}

generic
make_blackbox_struct (generic name) {
  generic rep = symbol ("rep");
  generic rep2= symbol ("rep2");
  generic vp  = Pointer (MMX_VOID);
  generic r1  = Type (rep, vp);
  generic pp  = Type (rep2, vp);
  generic r2  = Define (Type (name, Inline (Constructor (pp))),
			Begin (Assign (rep, rep2)));
  generic r3  = Define (Type (name, Inline (Destructor ())),
			Begin ());
  return make_struct (name, list (r1, r2, r3));
}

/******************************************************************************
* Names
******************************************************************************/

generic
extender::gentype_rep () {
  return gentype * symbol ("_rep");
}

generic
extender::type_rep (generic name) {
  return gentype * symbol ("_") * as_symbol (name) * symbol ("_rep");
}

generic
extender::genmethod (generic name) {
  return symbol ("gen_") * ccpp_protect (name);
}

generic
extender::geninst () {
  return gentype * symbol ("_inst");
}

generic
extender::geninst (int nr) {
  return gentype * symbol ("_inst_") * symbol (as_string (nr));
}

generic
extender::curtype (int nr) {
  return gentype * symbol ("_cur_") * symbol (as_string (nr));
}

generic
extender::concrete (int nr) {
  return symbol ("concrete_") * symbol (as_string (nr));
}

generic
extender::abstract (int nr) {
  return symbol ("abstract_") * symbol (as_string (nr));
}

/******************************************************************************
* Interface
******************************************************************************/

void
extender::produce () {
  int i;
  produce_gentype ();
  produce_geninst ();
  for (i=1; i<=max_type_args; i++)
    produce_geninst (i);
  produce_type (types);
}

void
extension_code (ostream& out, generic prog) {
  extender ext;
  ext.parse (prog);
  ext.produce ();
  ccpp_print (out, cons (MMX_BEGIN, reverse (ext.output)));
}

} // namespace mmx
