
/******************************************************************************
* MODULE     : extend.hh
* DESCRIPTION: Convert mathemagix trees to C or C++
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __EXTEND_HH
#define __EXTEND_HH
#include <symbols.hh>
#include <list.hh>
namespace mmx {

class extender {
private:
  list  types;
  table type_conv;
  list  exports;
  table export_env;
  list  imports;
  list  output;
  int   max_type_args;

  symbol  gentype;
  generic gentype_rep ();
  generic genmethod (generic name);
  generic geninst ();
  generic geninst (int nr);
  generic curtype (int nr);
  generic concrete (int nr);
  generic abstract (int nr);
  generic type_rep (generic name);

private:
  void    error (generic where, string message);
  void    warning (generic where, string message);
  void    generate (generic g);
  generic rewrite (generic g);
  void    type_decl (generic g);
  void    export_decl (generic g);
  generic specialize (generic g, generic to, bool virtual_flag);
  list    specialize (list l, generic to, bool virtual_flag);
  generic type_as_ccpp (generic g);
  generic decl_as_ccpp (generic g, bool gen_flag= false);
  list    decl_as_ccpp (list l, bool gen_flag= false);
  void    gentype_methods (generic g);
  void    gentype_methods (list l);
  void    produce_gentype ();
  void    produce_geninst ();
  list    abstract (int nr, list args, list types);
  void    geninst_methods (int nr, generic g);
  void    geninst_methods (int nr, list l);
  void    produce_geninst (int nr);
  list    filter (list l, list target_l);
  void    produce_atomic_type (generic mmx_name, generic ccpp_name);
  void    produce_template_type (symbol name, list args);
  void    produce_type (generic g);
  void    produce_type (list l);

public:
  inline extender (): max_type_args (0), gentype ("gentype") {}
  inline ~extender () {}

  void parse (generic prog);
  void produce ();

  friend void extension_code (ostream& out, generic prog);
};

inline symbol operator * (symbol s1, symbol s2) {
  return symbol (as_string (S (s1)) * as_string (S (s2))); }
void nice_print (generic text, list l);
void nice_print (symbol text, list l);

generic get_decl_var (generic g);
generic get_decl_type (generic g);
generic get_decl_val (generic g);
generic make_struct (generic name, list args, list inherit= list ());
generic make_blackbox_struct (generic name);

}
#endif // __EXTEND_HH
