
/******************************************************************************
* MODULE     : glue.cc
* DESCRIPTION: glue for external libraries
* COPYRIGHT  : (C) 2003  Bernard Mourrain
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <glue.hh>

namespace mmx {

string
get_string (generic g) {
  //  cerr << ">>> " << g << "\n";
  return as_string (as_symbol (g));
}

string& operator<<(string & s, const symbol& u)
{
  string x= as_string(S(u));
  s<<x(1,N(s)-1);
  return s;
} 


table env;

//string&  operator<<(,symbol);
//static int  counter= 0;

bool
is_operator(generic fct)
{
  return(fct == MMX_PLUS || fct == MMX_TIMES || fct== MMX_MINUS 
	 || fct== MMX_OVER || fct == MMX_DOT);
}

bool
is_new_type(generic g)
{
  if(!is_list(g)) return false;
  generic t = car(cdr(g));
  if(nil(t)) return false;
  return(car(cdr(cdr(t))) == symbol("Type"));
}

bool
is_new_function(generic g)
{
  if(!is_list(g)) return false;
  generic t = car(cdr(g));
  if(nil(t)) return false;
  return(car(cdr(cdr(t))) != symbol("Type"));
}
bool is_keyword(generic g)
{
  if(!is_list(g)) return false;
  return (nil(cdr(g)));
}
bool
is_include(generic g)
{
  if(car(g) != MMX_DEFINE) return false;
  generic t = car(cdr(g));
  if(!is_list(t) || nil(t)) return false;
  return(car(cdr(t)) == symbol("Include"));
}

bool
is_assign(generic g)
{
  //  std::cerr<<"AA "<<g<<std::endl;
  if(!is_list(g)) return false;
  g = cdr(g);
  if(nil(g)) return false;
  return (car(g) == symbol("assign"));
}
bool
is_variable(generic g)
{
  if(!is_list(g)) return false;
  g = cdr(g);
  if(nil(g)) return false;
  g = car(g);
  if(nil(g)) return false;
  g = cdr(g);
  if(nil(g)) return false;
  return (car(g) == symbol("Variable"));
}
bool is_init(generic g)
{
  if(car(g) != MMX_DEFINE) return false;
  generic t = car(cdr(g));
  if(!is_list(t) || nil(t)) return false;
  return(car(cdr(t)) == symbol("Init"));
}

bool is_functional(generic g)
{
  //  std::cerr<<"AA "<<std::endl;
  if(!is_list(g)) return false;
  g =cdr(g);
  if(!is_list(g)) return false;  
  g =cdr(g);
  if(!is_list(g)) return false;
  g =car(g);
  if(!is_list(g)) return false;
  return (car(g) == MMX_FUNCTION);
}
int find(string s, string u)
{
  int i, j;
  for(i=0;i<N(s);i++)
    {
      for(j=0; i+j<N(s) && j<N(u) && s[i+j]==u[j] ;j++);
      if(j==N(u)) return i;
    }
  return -1;
}
//----------------------------------------------------------------------
string
input () {
  cout << "> ";
  fflush (stdout);
  char in[1000];
  cin.getline (in, 1000);
  return in;
}

void
parse_string (string s) {
  list errors;
  parse (s, errors);
  print_errors (errors);
  //  cout << output << "\n";
}

//======================================================================
generic define_new_type(generic g)
{
  //  std::cerr<<">>> New Type  "<<g<<std::endl;
  generic s = car(cdr(car(cdr(g))));
  env(s)   =  S(as_literal(car(cdr(cdr(g)))));
  //  std::cerr<<">>> New Type  "<<env(s)<<std::endl;
  return s;
}
//======================================================================
void cpp::new_type(generic g)
{
  generic s=   define_new_type(g); 
  fout <<"mmx::symbol MMX_"<<s<<"(\""<<s<<"\"); "
       <<"mmx::mirror ID_"<<s<<"(MMX_"<<s<<");"<<std::endl;
}
void  cpp::include(generic g)
{
  generic l = car(cdr(cdr(g)));
  l = cdr(l);
  while(!nil(l))
    { 
      string s= as_string(S(as_symbol(car(l))));
      s=s(1,N(s)-1);
      fout <<"#include <"<<as_charp(s)<<">"<<std::endl;;
      l =cdr(l);
    }
  fout <<std::endl;
}

void generate_init(std::fstream & fout, generic g)
{
  generic l = car(cdr(cdr(g)));
  l = cdr(l);
  while(!nil(l))
    { 
      fout <<as_charp(get_string(car(l)))<<";"<<std::endl;;
      l =cdr(l);
    }
  fout <<std::endl; 
}


void cpp::instruction (generic v, generic out)
{
  //  cerr << ">>> Instruction " << v <<" "<< gentype (v) << lf;
  string s=  get_string(v);
  int i= find(s,"return ");
  if(i >= 0)
    {
      fout<<as_charp(s(0,i));
      s=s(i+7,N(s));
      int e=find(s,";");
      if(e>=0) s=s(0,e);
    }

  fout<<"return ";
  fout<<"mmx::mmx_obj< ";
  symbol y = as_symbol(env[out]); 
  fout << as_charp(as_string(y));
  fout <<" >(";
  fout<<as_charp(s);
  fout <<");"<<std::endl;

}
void  cpp::function_body(generic w, generic out)
{
  symbol v= S (as_literal (w));
  //  cerr << ">>> Function " << v << " "<< gentype (v) << lf;
  if(!is_symbol(v)) {
       fout <<"Syntax error not a string!"<< v<<std::endl;
  } else 
  {
      fout <<"  ";
      if(out == symbol("Void")) 
	{
	  fout<<as_charp(get_string(v));
	  fout <<";\n  return _g;\n";
	}
      else if(out == symbol("Variable")) 
	{
	  string s=  get_string(v);
	  int i= find(s,"return ");
	  if(i >= 0)
	    {
	      fout<<as_charp(s(0,i));
	      s=s(i+7,N(s));
	      int e=find(s,";");
	      if(e>=0) s=s(0,e);
	    }
	  
	  fout <<"return ("<<as_charp(s)<<")->ref;"<<std::endl;
	}
      else
	instruction(v,out);
  }
}
void cpp::nameoffunction(generic g)
{
  symbol s= as_symbol(g);
  if(s=="+")
    fout<<"add";
  else if(s=="-")
    fout<<"sub";
  else if(s=="*")
    fout<<"mul";
  else if(s=="/")
    fout<<"div";
  else if(s==".")
    fout<<"pnt";
  else
    fout<<s;
} 

void cpp::namefromvarsig(generic g)
{
  generic l = cdr(car(cdr(g))); 
  fout << "mmx_fct_";
  nameoffunction(car(car(cdr(g))));
  while(!nil(l))
    { 
      generic arg = cdr(car(l));
      generic typ = car(cdr(arg));
      fout<<"_"<< typ;
      l=cdr(l);
    }
  generic ot = car(cdr(cdr(g)));
  fout<<"__"<<ot;
}

void cpp::mmx_name(generic g)
{
  if(is_functional(g))
    if(is_assign(g))
       fout << "MMX_ASSIGN";
    else  if (car(cdr(g))== symbol("new"))
      {
        fout << "MMX_NEW_";
        generic l = car(cdr(cdr(g)));
        fout<<car(cdr(car(cdr(l))));
      }
    else
      fout<<"\""<<car(cdr(g))<<"\"";
  else
    fout<<car(car(cdr(g)));
}
void cpp::namefromfunctionalsig(generic g)
{

  fout << "mmx_fct_";
  nameoffunction(car(cdr(g)));
  generic l = car(cdr(cdr(g)));
  generic ot = car(cdr(cdr(l)));
  l = cdr(car(cdr(l)));
  while(!nil(l))
    { 
      generic typ = car(l);
      fout<<"_"<< typ;
      l=cdr(l);
    }
  fout<<"__"<<ot;
}

void cpp::namefromsig(generic g)
{
  if(is_functional(g))
    namefromfunctionalsig(g);
  else
    namefromvarsig(g);
}


void cpp::new_function(generic g)
{
  //  cerr << ">>> New function " << g << " "<< gentype (g) << lf;
  //  std::cout<<">>> "<<g<<std::endl;
  generic sig = car(cdr(g));
  generic val = car(cdr(cdr(g)));
  //  std::cout<<"| "<<sig<<" "<<val<<std::endl;
  fout <<"\n//"<<sig
       << "\nmmx::generic ";
  namefromsig(sig);
  fout<<"(mmx::generic _g)";
  //  std::cerr << ">>> Start body " << g << " " << gentype (g) << lf;
  int c;
  if(gentype (val)!=LIST_TYPE)
    {
      generic l = car(cdr(cdr(sig)));
      l = cdr(car(cdr(l))); 
      c=1;
      fout <<"\n{"<<std::endl;
      while(!nil(l))
	{ 
	  generic typ = car(l);
	  if(nil(env[typ]))
	  {
	    std::cerr <<"Error: "<<typ<<" not defined"<<std::endl;
	    return;
	  }
	  symbol x= as_symbol(env[typ]);
	  fout<< "  "<<as_charp(as_string(x))<<" x"<<c
	      <<"(mmx::view_as< "<<as_charp(as_string(x))
	      <<" >(_g["<<c<<"]));"<<std::endl;
	  l=cdr(l); c++;
	}
      l = car(cdr(cdr(sig)));
      symbol fct=as_symbol(val);
      symbol typ=as_symbol(env[car(cdr(cdr(l)))]);
      fout<<"  return mmx_obj< "<<as_charp(as_string(typ)) <<" >("
	  <<as_charp(as_string(fct))<<"(";
      l = cdr(car(cdr(l))); 
      if(!nil(l)) fout<<"x1";
      l=cdr(l);     c=2;
      while(!nil(l))
	{
	  fout<<",x"<<c;
	  l=cdr(l);
	  c++;
	}
      fout<<"));"<<std::endl;
    }
  else 
    {
      generic l = car(cdr(val)); int c=1;
      fout <<"\n{"<<std::endl;
      while(!nil(l))
	{ 
	  generic arg = cdr(car(l));
	  generic typ = car(cdr(arg));
	  //	  std::cerr<<env[typ]<<std::endl;
	  if(nil(env[typ]))
	  {
	    std::cerr <<typ<<" not defined"<<std::endl;
	    return;
	  }
	  symbol x= as_symbol(env[typ]);
	  fout<< "  "<<as_charp(as_string(x))<<" "<<car(arg)
	      <<"(mmx::view_as< "<<as_charp(as_string(x))
	      <<" >(_g["<<c<<"]));"<<std::endl;
	  l=cdr(l); c++;
	}
      sig = car(cdr(cdr(sig)));
      generic out;
      if(is_variable(sig)) 
	out= symbol("Variable");
      else
	out = car(cdr(cdr(sig)));
      //      std::cerr<<"A>> "<<sig<<" "<<val<<std::endl;
      val = car(cdr(cdr(cdr(val))));
      function_body(val,out);
    }
  fout <<"}"<<std::endl;
}


void cpp::external(generic g)
{
    string file=as_string(S(as_literal(car(g))));
    std::cout<<as_charp(file*".mmx.so ");
    file = file*string(".mml");
    //    std::cerr<<"Parsing file "<<file<<std::endl;
    mmx::string s;
    if (load (file, s)) 
      std::cout << "Couldn't open file "<<file<<"\n"; 
    else 
      {
	mmx::parse_string (s);
	mmx::list errors;
	mmx::generic output= mmx::parse (s, errors);
	//	std::cerr<<output<<std::endl;
	mmx::print_errors (errors);
	output  =  (mmx::cdr (mmx::car (mmx::cdr(output))));
	ext out(fout);
	out.generate(output);
      }
}

void cpp::init() 
{
  env(MMX_INT)=symbol(" int ");
  //  env(MMX_DOUBLE)=symbol(" double ");
  env(MMX_STRING)=symbol(" char* ");
  env(symbol("Void") )=symbol(" void ");
  env(symbol("LiteralFloat"))=symbol(" mmx::symbol ");
  env(symbol("Variable"))=symbol(" mmx::alias ");
  fout <<"#include <link.H>"<<std::endl;
  fout <<"#include <iostream>"<<std::endl;
  fout <<"#define MMX_Int    MMX_INT"<<std::endl;
  fout <<"#define ID_Int     ID_INT"<<std::endl;
  fout <<"#define MMX_String MMX_STRING"<<std::endl;
  fout <<"#define ID_String  ID_STRING"<<std::endl;
  fout <<"#define MMX_Void   MMX_VOID"<<std::endl;
  fout <<"#define ID_Void    ID_VOID"<<std::endl;
  fout <<"#define ID_LiteralFloat  ID_LITERAL_FLOAT"<<std::endl;
  fout <<"#define MMX_NEW_LiteralFloat MMX_NEW_FLOAT"<<std::endl;	
}

void cpp::generate (generic g) 
{
  list l = as_list(g);
  //  std::cerr<<"> "<<l<<std::endl;
  while(!nil(l))
    {
      generic v=car(l);
      //      std::cerr<<">>> "<<v<<std::endl;
      if (!is_list(v));
      else if(as_symbol(car(v)) == MMX_BEGIN)
	  generate(cdr(v));
      else if(as_symbol(car(v)) == MMX_LOCAL)
	  generate(cdr(v));
      else if(as_symbol(car(v)) == symbol("use"))
	  external(cdr(v));
      else if(as_symbol(car(v))==symbol ("preamble")) {
	generic arg= v[1];
	if (is_literal (arg)) arg= S (as_literal (g));
	if (is_symbol  (arg)) arg= S (as_symbol  (g));
	if (is_string  (arg)) arg= unquote (as_string (g));
	raw_print (fout, as_string (arg));
	// NOTE: now uses /"..."/ syntax
      }
      else if(is_new_type(v))
	new_type(v);
      else if(is_new_function(v))
	new_function(v);
//       else
// 	fout<<g;
      l=cdr(l);
    }
}

//======================================================================
void declare::generate(generic g)
{
  list l = as_list(g);
  //  std::cerr<<l<<std::endl;
  while(!nil(l))
    {
      generic v=car(l);
      //std::cerr<<">> "<<v<<std::endl;
      if (!is_list(v));
      else if(as_symbol(car(v)) == MMX_BEGIN)
	generate(cdr(v));
      else if(as_symbol(car(v)) == MMX_LOCAL)
	generate(cdr(v));
      else if(as_symbol(car(v)) == symbol("use"))
	external(cdr(v));
      else if(as_symbol(car(v))==symbol("preamble"));
      else if(is_new_type(v))
	new_type(v);
      else if(is_new_function(v))
	new_function(v);
//       else	fout<<g;
      l=cdr(l);
    }
}

void declare::init(string arg)
{

   fout <<"\nextern \"C\" void init_"<<as_charp(arg)<<"()\n{\n"
	<<"  using namespace mmx;"<<std::endl;
}

void declare::close(string arg)
{
//   fout <<"  load_"<<as_charp(arg)<<"=true;"<<std::endl;
//   fout<< env(symbol(string("mmx_module")*arg))= true;
  fout<<"}"<<std::endl;
}

void declare::new_type(generic g)
{
  generic typ=car(cdr(car(cdr(g))));
  fout <<"  new_type(MMX_"<<typ<<", ID_"<<typ<<");\n"<<std::endl;;
}

void declare::new_function(generic g)
{
  generic sig = car(cdr(g));
  fout <<"  new_function(";
  mmx_name(sig);
  fout<<",";
  namefromsig(sig); 
  fout<<",mmx::function_type(";
  if(is_functional(sig))
    {
      //      std::cerr<<"B>> "<<sig<<std::endl;
      generic l = car(cdr(cdr(sig)));
      generic ot = car(cdr(cdr(l)));
      l = cdr(car(cdr(l)));
      if(!nil(l))
	{
	  if(is_assign(sig) && !nil(l))
	    fout << "list(ID_ALIAS, ID_"<<ot<<"),";
	  else
	    fout << "ID_"<<car(l)<<",";
	  l=cdr(l);
	  while(!nil(l))
	    { fout<< "ID_"<<car(l)<<","; l=cdr(l);}
	}
      fout << "ID_"<<ot<<"));"<<std::endl;
    }
  else {
    //    std::cerr<<"C>> "<<sig<<std::endl;
    generic l = cdr(car(cdr(sig))); 
    while(!nil(l))
      { fout<< "ID_"<<car(cdr(cdr(car(l))))<<","; l=cdr(l);}
    fout << "ID_"<<car(cdr(cdr(sig)))<<"));"<<std::endl;
  }
}

void declare::external(generic g)
{
  string name=as_string(S(as_literal(car(g))));
  //  fout <<"  if(!load_"<<as_charp(name)<<") "
  fout<< "  mmx::load_external("<<name<<");"<<std::endl;
}
//======================================================================
void ext::generate(generic g)
{
   list l = as_list(g);
   //  std::cerr<<">> "<<l<<std::endl;
   while(!nil(l))
     {
       generic v=car(l);
      //std::cerr<<">> "<<v<<std::endl;
      if (!is_list(v));
      else if(as_symbol(car(v)) == MMX_BEGIN)
	  generate(cdr(v));
      else if(as_symbol(car(v)) == MMX_LOCAL)
	  generate(cdr(v));
      else if(as_symbol(car(v)) == symbol("use"))
	  external(cdr(v));
      else if(as_symbol(car(v))==symbol("preamble"));
      else if(is_new_type(v))
	new_type(v);
      else if(is_new_function(v));
//       else
// 	fout<<g;
      l=cdr(l);
     }
}
//----------------------------------------------------------------------
void ext::new_type(generic g)
{
  generic s=   define_new_type(g); 
  fout <<"extern mmx::mirror ID_"<<s<<";"<<std::endl;
} 
//======================================================================
} // namespace mmx
