
/******************************************************************************
* MODULE     : file.cc
* DESCRIPTION: Loading and saving files
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <file.hh>
#include <list.hh>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
using std::ifstream;
using std::ofstream;
namespace mmx {

/******************************************************************************
* Location handling
******************************************************************************/

static int  cur_location = 0;
static list sources;

void
insert_source (string& src, string name) {
  src->begin= cur_location;
  cur_location += N(src);
  src->end= cur_location;
  cur_location ++;
  sources= cons (list (string (name), src), sources);
}

void
find_location (int location, string& src, string& name,
	       int& row, int& col)
{
  list l= sources;

  while (!nil (l)) {
    src= as_string (car (l) [1]);
    if ((location >= src->begin) && (location <= src->end)) {
      int i, pos= location - src->begin;
      name= as_string (car (l) [0]);
      for (i=0, row=1, col=1; i<pos; i++)
	if (src[i]=='\n') { row++; col=1; }
	else col++;
      return;
    }
    l= cdr (l);
  }
 
  src = "";
  name= string ("unknown");
  row = 0;
  col = 0;
}

/******************************************************************************
* System commands
******************************************************************************/

static string sys_home= "";
static string sys_load_path= "";

string
get_env (string var) {
  char* _var= as_charp (var);
  char* _ret= getenv (_var);
  delete[] _var;
  if (_ret==NULL) return "";
  string ret (_ret);
  return ret;
  // do not delete _ret !
}

int
system (string s) {
  char* _s= as_charp (s);
  int r= std::system (_s);
  delete[] _s;
  return r;
}

static void
init_system () {
  if (sys_home == "") {
    sys_home= get_env ("HOME");
    system ("mkdir -p " * sys_home * "/.mathemagix");
    system ("mkdir -m 744 -p " * sys_home * "/.mathemagix/system");
    system ("mkdir -m 744 -p " * sys_home * "/.mathemagix/mmx");
    sys_load_path= get_env ("MMX_LOAD_PATH");
    if (sys_load_path == "")
      sys_load_path= sys_home * "/.mathemagix/mmx";
    else sys_load_path= sys_load_path * ":" * sys_home * "/.mathemagix/mmx";
    sys_load_path= sys_load_path * ":" * MMXDATA * "/mmx/library";
    sys_load_path= sys_load_path * ":" * MMXDATA * "/mmx/examples";
    sys_load_path= sys_load_path * ":" * MMXSRC * "/mmx/library";
    sys_load_path= sys_load_path * ":" * MMXSRC * "/mmx/examples";
  }
}

string
eval_system (string s) {
  init_system ();
  string temp= sys_home * "/.mathemagix/system/temp";
  system (s * " > " * temp);
  string r;
  bool flag= load (temp, r, false);
  system ("rm " * temp);
  if (flag) return "";
  while ((N(r)>0) && (r[N(r)-1]=='\n')) r= r (0, N(r)-1);
  return r;
}

/******************************************************************************
* Loading and saving
******************************************************************************/

bool
load (string file_name, string& s, bool decode) {
  if (decode) file_name= eval_system ("echo " * file_name);
  ifstream f1 (as_charp (file_name));
  if (!f1) return true;
  else {
    char c;
    s= string ("");
    while (f1.get (c)) s << c;
    insert_source (s, file_name);
    return false;
  }
}

bool
load (string file_path, string file_name, string& s, bool decode) {
  if (decode) file_name= eval_system ("echo " * file_name);
  if ((N(file_name) > 0) && (file_name[0] == '/'))
    return load (file_name, s, false);
  int i, n= N(file_path), start= 0;
  for (i=0; i<=n; i++)
    if ((i==n) || (file_path[i] == ':')) {
      string ss= file_path (start, i);
      if ((N(ss) > 0) && (ss[0] == '$')) {
	init_system ();
	if (ss == "$MMX_LOAD_PATH") ss= sys_load_path;
	else ss= get_env (ss (1, N(ss)));
	if (!load (ss, file_name, s, false)) return false;
      }
      else {
	if (ss == "") ss= ".";
	if (ss[N(ss)-1] != '/') ss= ss * "/";
	if (!load (ss * file_name, s, false)) return false;
      }
      start= i+1;
    }
  return true;
}

bool
save (string file_name, string s, bool decode) {
  if (decode) file_name= eval_system ("echo " * file_name);
  ofstream f1 (as_charp (file_name));
  if (!f1) return true;
  else {
    int i, n=N(s);
    for (i=0; f1 && (i<n); i++) f1.put (s[i]);
    return false;
  }
}

} // namespace mmx
