
/******************************************************************************
* MODULE     : symbols.hh
* DESCRIPTION: frequently used symbols and values
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLS_HH
#define __SYMBOLS_HH
#include <symbol.hh>
#include <mirror.hh>
#include <value.hh>
#include <list.hh>
namespace mmx {

/******************************************************************************
* Mathemagix keyword names
******************************************************************************/

extern symbol MMX_ACCESS;
extern symbol MMX_AMBIGUOUS;
extern symbol MMX_AND;
extern symbol MMX_ASSIGN;
extern symbol MMX_BACKPRIME;
extern symbol MMX_BEGIN;
extern symbol MMX_BREAK;
extern symbol MMX_CASE;
extern symbol MMX_COMPLEMENT;
extern symbol MMX_COMPOSE;
extern symbol MMX_CONTINUE;
extern symbol MMX_CROSS;
extern symbol MMX_DECLARE;
extern symbol MMX_DEFINE;
extern symbol MMX_DEFINEALIAS;
extern symbol MMX_DOLLAR;
extern symbol MMX_DOT;
extern symbol MMX_DOTS;
extern symbol MMX_ELSE;
extern symbol MMX_EQUAL;
extern symbol MMX_EQUIV;
extern symbol MMX_ERROR;
extern symbol MMX_EXISTS;
extern symbol MMX_EXISTSVAL;
extern symbol MMX_EXPORT;
extern symbol MMX_FACTORIAL;
extern symbol MMX_FOR;
extern symbol MMX_FORALL;
extern symbol MMX_FORALLVAL;
extern symbol MMX_FORIN;
extern symbol MMX_FUNCTION;
extern symbol MMX_GENERATE;
extern symbol MMX_GTR;
extern symbol MMX_GTREQ;
extern symbol MMX_GTRGTR;
extern symbol MMX_HAS;
extern symbol MMX_HOLDS;
extern symbol MMX_IF;
extern symbol MMX_JOIN;
extern symbol MMX_IMPLIES;
extern symbol MMX_IMPORT;
extern symbol MMX_LAMBDA;
extern symbol MMX_LESS;
extern symbol MMX_LESSEQ;
extern symbol MMX_LESSLESS;
extern symbol MMX_LESSLESS_VAR;
extern symbol MMX_LESSLESS_STR;
extern symbol MMX_LOAD;
extern symbol MMX_LOCAL;
extern symbol MMX_LOOP;
extern symbol MMX_MIGRATE;
extern symbol MMX_MINUS;
extern symbol MMX_MINUSASSIGN;
extern symbol MMX_MODIFY;
extern symbol MMX_NEW_INT;
extern symbol MMX_NEW_FLOAT;
extern symbol MMX_NEW_STRING;
extern symbol MMX_NOT;
extern symbol MMX_NOTGTR;
extern symbol MMX_NOTGTREQ;
extern symbol MMX_NOTLESS;
extern symbol MMX_NOTLESSEQ;
extern symbol MMX_NUMBER;
extern symbol MMX_OBTAIN;
extern symbol MMX_OMINUS;
extern symbol MMX_OOVER;
extern symbol MMX_OPLUS;
extern symbol MMX_OR;
extern symbol MMX_OTIMES;
extern symbol MMX_OVER;
extern symbol MMX_OVERASSIGN;
extern symbol MMX_PLUS;
extern symbol MMX_PLUSASSIGN;
extern symbol MMX_POSTDEC;
extern symbol MMX_POSTINC;
extern symbol MMX_POWER;
extern symbol MMX_PREDEC;
extern symbol MMX_PREINC;
extern symbol MMX_PRIME;
extern symbol MMX_RETRIEVE;
extern symbol MMX_RETURN;
extern symbol MMX_SEQAND;
extern symbol MMX_SEQOR;
extern symbol MMX_SPLIT;
extern symbol MMX_SQTUPLE;
extern symbol MMX_STEP;
extern symbol MMX_TIMES;
extern symbol MMX_TIMESASSIGN;
extern symbol MMX_TRANSTYPE;
extern symbol MMX_TUPLE;
extern symbol MMX_TUPLETYPE;
extern symbol MMX_TYPE;
extern symbol MMX_UNEQUAL;
extern symbol MMX_UNTIL;
extern symbol MMX_USE;
extern symbol MMX_VTUPLE;
extern symbol MMX_WHILE;
extern symbol MMX_XOR;
extern symbol MMX_YIELD;

/******************************************************************************
* Mathemagix types
******************************************************************************/

extern symbol MMX_CLASS;
extern symbol MMX_GENERIC;
extern symbol MMX_BOOL;
extern symbol MMX_INT;
extern symbol MMX_STRING;
extern symbol MMX_OSTREAM;
extern symbol MMX_OSTRSTREAM;
extern symbol MMX_LITERAL_INT;
extern symbol MMX_LITERAL_FLOAT;
extern symbol MMX_LITERAL_STRING;
extern symbol MMX_ALIAS;
extern symbol MMX_GENERATOR;
extern symbol MMX_LIST;
extern symbol MMX_ARRAY;
extern symbol MMX_TABLE;
extern symbol MMX_POINTER;

extern mirror ID_CLASS;
extern mirror ID_GENERIC;
extern mirror ID_BOOL;
extern mirror ID_INT;
extern mirror ID_STRING;
extern mirror ID_OSTREAM;
extern mirror ID_OSTRSTREAM;
extern mirror ID_LITERAL_INT;
extern mirror ID_LITERAL_FLOAT;
extern mirror ID_LITERAL_STRING;
extern mirror ID_ALIAS;
extern mirror ID_GENERATOR;
extern mirror ID_LIST;
extern mirror ID_ARRAY;
extern mirror ID_TABLE;
extern mirror ID_POINTER;
extern mirror ID_CROSS;
extern mirror ID_TUPLE;
extern mirror ID_FUNCTION;
extern mirror ID_FORALL;
extern mirror ID_EXISTS;
extern mirror ID_IMPLIES;
extern mirror ID_AND;
extern mirror ID_OR;
extern mirror ID_NOT;

extern value VAL_CLASS;
extern value VAL_GENERIC;
extern value VAL_BOOL;
extern value VAL_INT;
extern value VAL_STRING;
extern value VAL_OSTREAM;
extern value VAL_OSTRSTREAM;

/******************************************************************************
* Miscellaneous
******************************************************************************/

extern symbol CONV_FUNCTION;
extern symbol CONV_ARGUMENT;
extern symbol CONV_AMBIGUOUS;
extern symbol CONV_APPLY;
extern symbol CONV_CROSS;
extern symbol CONV_FORALL;
extern symbol CONV_IGNORE;
extern symbol CONV_LAMBDA;
extern symbol CONV_TEMPLATE;
extern symbol CONV_UNALIAS;
extern symbol CONV_UNAND;
extern symbol CONV_UNCROSS;

extern symbol CONT_BREAK;
extern symbol CONT_RETURN;
extern symbol CONT_YIELD;
extern symbol MISC_DONE;
extern symbol MISC_GENERATE;
extern symbol MISC_SAVE;
extern symbol MISC_TODO;
extern symbol MISC_YIELD;

extern symbol  MMX_VOID;
extern generic ID_VOID;
extern value   VAL_VOID;
extern value   VAL_NONE;
extern symbol  MMX_EXCEPTION;
extern generic ID_EXCEPTION;
extern value   VAL_EXCEPTION;
extern value   VAL_DONE;
extern generic ID_UNIVERSAL;

/******************************************************************************
* Shortcuts
******************************************************************************/

inline generic Assign (generic g1, generic g2) {
  return list (MMX_ASSIGN, g1, g2); }
inline generic Begin () {
  return list (MMX_BEGIN); }
inline generic Begin (generic g1) {
  return list (MMX_BEGIN, g1); }
inline generic Begin (generic g1, generic g2) {
  return list (MMX_BEGIN, g1, g2); }
inline generic Begin (list l) {
  return cons (MMX_BEGIN, l); }
inline generic Begin (generic g1, list l) {
  return cons (MMX_BEGIN, g1, l); }
inline generic Define (generic g1, generic g2) {
  return list (MMX_DEFINE, g1, g2); }
inline generic Dot (generic g1, generic g2) {
  return list (MMX_DOT, g1, g2); }
inline generic Return (generic g1) {
  return list (MMX_RETURN, g1); }
inline generic Type (generic g1, generic g2) {
  return list (MMX_TYPE, g1, g2); }

inline generic Function (generic g1, generic g2) {
  return list (MMX_FUNCTION, g1, g2); }
inline generic Pointer (generic g1) {
  return list (MMX_POINTER, g1); }

}
#endif // __SYMBOLS_HH
