
/******************************************************************************
* MODULE     : symbol.hh
* DESCRIPTION: symbols
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOL_HH
#define __SYMBOL_HH
#include <string.hh>
#include <table.hh>
namespace mmx {

class symbol;
class symbol_rep: public generic_rep {
protected:
  int      hash_value;
  generic* ref;

  int Type ();
  int Hash ();
  bool Equal (generic x);
  generic Copy ();
  void Print (ostream& out);

public:
  symbol_rep (generic g, int begin=-1, int end=-1);

  friend class symbol;
  friend generic& S (symbol s);
  friend ostream& operator << (ostream& out, symbol s);
  friend bool is_int (symbol s);
  friend bool is_double (symbol s);
  friend int as_int (symbol s);
  friend double as_double (symbol s);
};

class symbol {
protected:
  symbol_rep* rep;
  inline symbol (symbol_rep* rep2): rep (rep2) {}

public:
  symbol (char* s, int begin=-1, int end=-1);
  symbol (string s, int begin, int end);
  symbol (generic g, int begin, int end);
  symbol (char* s, generic where);
  symbol (string s, generic where);
  symbol (generic g, generic where);
  symbol (string s);
  symbol (generic g);
  inline friend symbol as_symbol (generic g) {
    if (gentype(g) != SYMBOL_TYPE) type_mismatch (g, "symbol");
    return symbol ((symbol_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline symbol_rep* operator -> () { return rep; }

  inline bool operator == (symbol s) { return rep->ref == s->ref; }
  inline bool operator != (symbol s) { return rep->ref != s->ref; }
  inline friend symbol copy (symbol s) { return symbol (s.rep); }
  inline friend generic& S (symbol s) { return *(s->ref); }
  friend ostream& operator << (ostream& out, symbol s);
  friend bool is_int (symbol s);
  friend bool is_double (symbol s);
  friend int as_int (symbol s);
  friend double as_double (symbol s);

  friend class symbol_rep;
};

inline bool
is_func (generic g, symbol s) {
  return is_list (g) && (N(g) > 0) && (car(g) == s);
}

inline bool
is_func (generic g, symbol s, int n) {
  return is_list (g) && (N(g) == (n+1)) && (car(g) == s);
}

}
#endif // __SYMBOL_HH
